/******************************************************************************
 *
 * Name: John Eisbrener
 * E-Mail: jmeisbre@mtu.edu
 * Exercise: 2
 * File: draw.c
 *
 * Note: This file is my addition to the program render that reads in a 
 *  winged-edge data structure into dynamical multidimensional arrays.  The
 *  program then traverses these arrays to make an object in OpenGL.
 *
 *****************************************************************************/

#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>

GLint vertex, face, edge;              /* global variables for vertex, face, &
                                          edge counts. */
GLdouble **vertex_coor;                /* pointer for the multidimensional 
                                          array for the vertex coordinates */
GLint *vertex_table;                   /* pointer for dynamic array for the
                                          vertex table */
GLint *face_table;                     /* pointer for dynamic array for the 
                                          face table */
GLint **edge_table;                    /* pointer for multidimensional array
                                          for the edge table */

/***** function declaration *****/
void draw_triangles(int, GLdouble [][3][3]);

/***** global variable *****/
GLdouble data[2][3][3] = { {{2.3, 2.3, 2.4}, {9.2, 9.5, 9.8}, {8.5, 8.9, 1.2}},
			 {{9.2, 9.5, 9.8}, {8.5, 8.9, 1.2}, {-7.1, 7.1, 7.1}} 
			};

/*** You own rendering operation goes here ***/
void cs390_draw()
{
  int i, j, edge_num, initial_edge, current_vertex, prev_vertex, side, flip;

  /* Draw the vertices */
  glColor4f(0.0, 0.0, 0.0, 1.0);
  glPointSize(5.0);                  /* Set the point size to 5.0 */
  glBegin(GL_POINTS);
     for(i = 0; i < vertex; i++)
       {
         glVertex3dv(vertex_coor[i]);
       }
  glEnd();
  /* End drawing the vertices */


  /* Draw the outline */
  for(i = 0; i < face; i++)
    {
      glColor4f(0.7, 0.7, 0.0, 1.0); /* yellow */
      glLineWidth(3);

      /* Draw outline of Polygon */
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
      glBegin(GL_POLYGON);
         edge_num = face_table[i];  /* Get the current edge */
         initial_edge = edge_num;   /* Set the initial edge */

         edge_num = edge_num - 1;   /* Get rid of off-by-one error */

         /* Check to see if you need to draw the first line backwards or not,
            the reason for this is because there is always one point that the
            face is on the wrong side of, so this counteracts that. */
         /* Draw the line backwards */
         if(edge_table[edge_num][2] == (i+1))
           {
             /* Draw first vertex */
             glVertex3dv(vertex_coor[ (edge_table[edge_num][1]-1) ]);
             /* Set current_vertex so you don't draw the same vertex twice */
             current_vertex = edge_table[edge_num][1];
           }
         else /* Draw the line normally */
           {
             /* Draw first vertex */
             glVertex3dv(vertex_coor[ (edge_table[edge_num][0]-1) ]);
             /* Set current_vertex so you don't draw the same vertex twice */
             current_vertex = edge_table[edge_num][0];
           }

         while(1)
           {
             /* You've already drawn this vertex */
             if(edge_table[edge_num][0] == current_vertex ||
                edge_table[edge_num][0] == prev_vertex)
               {
                 glVertex3dv(vertex_coor[ (edge_table[edge_num][1])-1 ]);

                 /* Set previous vertex to last vertex drawn */
                 prev_vertex = current_vertex;
                 /* Set current vertex drawn */
                 current_vertex = edge_table[edge_num][1];
               }
             else /* You haven't drawn this vertex */
               {
                 glVertex3dv(vertex_coor[ (edge_table[edge_num][0])-1 ]);

                 /* Set previous vertex to last vertex drawn */
                 prev_vertex = current_vertex;
                 /* Set current vertex drawn */
                 current_vertex = edge_table[edge_num][0];
               }

             /* Locate the correct side the face is on */
             if(edge_table[edge_num][2] == (i+1))
               {
                 side = 5;  /* face is on the left side (array index is 5) */
               }
             else
               {
                 side = 7;  /* face is on the right side (array index is 7) */
               }

             /* Check to see if the face is completely drawn. */
             if(initial_edge == edge_table[edge_num][side])
               {
                 break;
               }

             edge_num = edge_table[edge_num][side];
             edge_num = edge_num - 1; /* Get rid of off-by-one error */

           }

      glEnd();
    }
  /* End of drawing the outline*/


  /* Draw the filled face */
  for(i = 0; i < face; i++)
    {
      glColor4f(0.0, 0.0, 0.7, 1.0); /* blue */
      glLineWidth(3);

      /* Draw outline of Polygon */
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
      glBegin(GL_POLYGON);
         edge_num = face_table[i];  /* Get the current edge */
         initial_edge = edge_num;   /* Set the initial edge */

         edge_num = edge_num - 1;   /* Get rid of off-by-one error */

         /* Check to see if you need to draw the first line backwards or not,
            the reason for this is because there is always one point that the
            face is on the wrong side of, so this counteracts that. */
         /* Draw the line backwards */
         if(edge_table[edge_num][2] == (i+1))
           {
             /* Draw first vertex */
             glVertex3dv(vertex_coor[ (edge_table[edge_num][1]-1) ]);
             /* Set current_vertex so you don't draw the same vertex twice */
             current_vertex = edge_table[edge_num][1];
           }
         else /* Draw the line normally */
           {
             /* Draw first vertex */
             glVertex3dv(vertex_coor[ (edge_table[edge_num][0]-1) ]);
             /* Set current_vertex so you don't draw the same vertex twice */
             current_vertex = edge_table[edge_num][0];
           }

         while(1)
           {
             /* You've already drawn this vertex */
             if(edge_table[edge_num][0] == current_vertex ||
                edge_table[edge_num][0] == prev_vertex)
               {
                 glVertex3dv(vertex_coor[ (edge_table[edge_num][1])-1 ]);

                 /* Set previous vertex to last vertex drawn */
                 prev_vertex = current_vertex;
                 /* Set current vertex drawn */
                 current_vertex = edge_table[edge_num][1];
               }
             else /* You haven't drawn this vertex */
               {
                 glVertex3dv(vertex_coor[ (edge_table[edge_num][0])-1 ]);

                 /* Set previous vertex to last vertex drawn */
                 prev_vertex = current_vertex;
                 /* Set current vertex drawn */
                 current_vertex = edge_table[edge_num][0];
               }

             /* Locate the correct side the face is on */
             if(edge_table[edge_num][2] == (i+1))
               {
                 side = 5;  /* face is on the left side (array index is 5) */
               }
             else
               {
                 side = 7;  /* face is on the right side (array index is 7) */
               }

             /* Check to see if the face is completely drawn. */
             if(initial_edge == edge_table[edge_num][side])
               {
                 break;
               }

             edge_num = edge_table[edge_num][side];
             edge_num = edge_num - 1; /* Get rid of off-by-one error */

           }

      glEnd();
    }
  /* End of drawing the filled face*/


  /* This is code that I will not use.
   * glColor4f(0.0, 0.0, 0.32, 1.0);
   * glLineWidth(3);
   * draw_triangles(2, data);
   * glColor4f(1.0, 1.0, 0.0, 1.0);
   * glutWireTorus(20.0, 50.0, 10, 20);
   * glColor4f(1.0, 0.0, 0.0, 1.0);
   * glTranslatef(30.0, 0.0, 1.0);
   * glutSolidCube(20.5);
   * This is the end of the code that was already here. */
}

/*** Your initialization goes here ***/
/*** for example, read the data from the file ***/
/* I obtained the code for multidimensional arrays on the internet located at:
 * http://www.eskimo.com/~scs/C-faq/q6.16.html */
/*** "argc", "argv" are actual command line arguments passed in ***/
void cs390_init(int argc, char **argv)
{
  FILE *file;
  int i, j;
  char buf[33];

  if(argc > 1)  /* We load in the file located in the argv string */
    {
      file = fopen(argv[1], "r");
      if(!file)  /* Check to see if the file exists */
        {
          printf("Open File Failed: File \"%s\" does not exist.\n", argv[1]);
          exit(1);
        }

      fscanf(file, "%s", buf); /* read from file */
      /* set the number of vertices */
      vertex = atoi(buf);

      /* allocate the memory used for the 2D array for the vertices */
      vertex_coor = (GLdouble **)malloc(vertex * sizeof(GLdouble *));
      for(i = 0; i < vertex; i++)
        {
          vertex_coor[i] = (GLdouble *)malloc(3 * sizeof(GLdouble));
        }

      /* read in the floats for the x,y,z coordinates */
      for(i = 0; i < vertex; i++)
        {
          for(j = 0; j < 3; j++)
            {
              fscanf(file, "%s", buf);
              /* set the coord. for x, y, z */
              vertex_coor[i][j] = atof(buf);
            }
        }

      /* allocate the memory used for the vertex table */
      vertex_table = (GLint *)malloc(vertex * sizeof(GLint));

      /* read in the vertex incident edges */
      for(i = 0; i < vertex; i++)
        {
          fscanf(file, "%s", buf);
          /* set the incident edge for the vertex table */
          vertex_table[i] = atoi(buf);
        }

      /* read the number of faces */
      fscanf(file, "%s", buf);
      face = atoi(buf);

      /* allocate the memory used for the face table */
      face_table = (GLint *)malloc(face * sizeof(GLint));

      /* read in the face incident edges */
      for(i = 0; i < face; i++)
        {
          fscanf(file, "%s", buf);
          /* set the incident edge for the face table */
          face_table[i] = atoi(buf);
        }

      /* read in the number of edges */
      fscanf(file, "%s", buf);
      edge = atoi(buf);

      /* allocate the memory used for the 2D array for the vertices */
      edge_table = (GLint **)malloc(edge * sizeof(GLint *));
      for(i = 0; i < edge; i++)
        {
          edge_table[i] = (GLint *)malloc(8 * sizeof(GLint));
        }

      /* read in the winged-edge data structure */
      for(i = 0; i < edge; i++)
        {
          for(j = 0; j < 8; j++)
            {
              fscanf(file, "%s", buf);
              /* set the values for the winged-edge data structure */
              edge_table[i][j] = atoi(buf);
            }
        }
    }
  else
    {
      printf("File not specified. Cannot Execute Program.\n");
      exit(0);
    }
}

/*** Primitivie which draws a set of triangles ***/
void draw_triangles(int n, GLdouble data[][3][3])
{
    int i;

    glBegin(GL_TRIANGLES);
	for(i = 0; i < n; i++) {
	    glVertex3dv(&data[i][0][0]);
	    glVertex3dv(&data[i][1][0]);
	    glVertex3dv(&data[i][2][0]);
	}
    glEnd();
}

